/*
 * OPALE is a scientific library under LGPL. Its main goal is to
 * develop mathematical tools for any scientist.
 *
 * Copyright (C) 2002 Opale Group
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * You can visit the web site http://opale.tuxfamily.org to obtain more
 * informations about this program and/or to contact the authors by mail
 * developers@opale.tuxfamily.org.
 */

package opale.ode.scheme;

import opale.tools.*;
import opale.mathtools.*;
import opale.ode.*;


/**
* Cette classe implmente le schma d'Euler implicite qui consiste  avancer en temps de la facon suivante : yn+1=yn+f(t,yn+1).
* @author O.C.
* @since Opale-ODE 0.11
*/
public  final class ImplicitEuler extends TimeScheme
{
/**
* This value represents the precision for the fix point method.
*/
//public static double PRECISION = 1.e-16;

/**
* This value fixes the maximum number of iterations in the algorithm.
*/ 
//public static int MAXITERATION = 100;
private DVect sol;

public void init()
{
super.init();
sol = new DVect(getPb().dim());
f.tmp = new DVect(getPb().dim());
}
	
/**
* Cette mthode dfinit l'avanc en temps par la formule d'Euler implicite.
*/	
public void forward(Equation eq)
	{
	int i=0,sortie=0;
	double cvg;

	sol.set(present());	
	f.eq=eq;
	
	boolean cv = FixPoint.solve(f,present().size(),present(), sol);
	if (opale.tools.Debug.On) opale.tools.Debug.print("Pt fixe cvg = "+cv);	
	if (cv == false) 
		Stdio.printerrln("WARNING!! The fix point method doesn't converge for the fully implicit Euler scheme");
	//initialisation des conditions initiales

	present().set(sol);	
	
	}
/**
* Cette mthode dfinit l'avanc en temps par la formule d'Euler implicite.
*/	
public void advance()
	{
	int i=0,sortie=0;
	double cvg;

	sol.set(past());	
	f.eq=getEquation();
	
	boolean cv = FixPoint.solve(f,present().size(),past(), sol);
	if (opale.tools.Debug.On) opale.tools.Debug.print("Pt fixe cvg = "+cv);	
	if (cv == false) 
		Stdio.printerrln("WARNING!! The fix point method doesn't converge for the fully implicit Euler scheme");
	//initialisation des conditions initiales
/*	DVect x = new DVect(present());	
	DVect sol = new DVect(present());	
	
	
	
	while ( (i<MAXITERATION) && (sortie == 0) )
		{
		//x.get(tab);
		sol.set(eq.derivs(time(),x));
		sol.mul(dt());
		sol.add(present());
		
		x.sub(sol);
		cvg =   x.norm2Sq()/sol.norm2Sq();
		
		// prepare the next iteration
		x.set(sol);
		if ( cvg < PRECISION ) 
			sortie = 1;
		i++;
		}
	
	present().set(sol);*/
	present().set(sol);	
	
	}

public String toString()
	{
	return "Implicit Euler";
	}


private class func implements IFnn
	{
	DVect tmp;
	Equation eq;
	
	public  DVect val(DVect x)
		{
		tmp.set(past());
		tmp.acc(dt(),eq.derivs(time(),x));
		return tmp;
		}
	} 
private func f =new func();

}
